@@ -7,6 +7,7 @@ import os
 
 from typing import (Any, AnyStr, Dict, List, Sequence, Text, Union, cast)
 
+from . import loghandler
 from schema_salad.ref_resolver import file_uri
 from .process import (Process, shortname)
 from .resolver import ga4gh_tool_registries
@@ -14,30 +15,29 @@ from .software_requirements import (SOFTWARE_REQUIREMENTS_ENABLED)
 
 _logger = logging.getLogger("cwltool")
 
-defaultStreamHandler = logging.StreamHandler()
-_logger.addHandler(defaultStreamHandler)
-_logger.setLevel(logging.INFO)
+DEFAULT_TMP_PREFIX = "tmp"
 
 
 def arg_parser():  # type: () -> argparse.ArgumentParser
-    parser = argparse.ArgumentParser(description='Reference executor for Common Workflow Language')
+    parser = argparse.ArgumentParser(
+        description='Reference executor for Common Workflow Language standards.')
     parser.add_argument("--basedir", type=Text)
     parser.add_argument("--outdir", type=Text, default=os.path.abspath('.'),
                         help="Output directory, default current directory")
 
-    parser.add_argument("--no-container", action="store_false", default=True,
-                        help="Do not execute jobs in a Docker container, even when specified by the CommandLineTool",
-                        dest="use_container")
-
-    parser.add_argument("--preserve-environment", type=Text, action="append",
-                        help="Preserve specific environment variable when running CommandLineTools.  May be provided multiple times.",
-                        metavar="ENVVAR",
-                        default=["PATH"],
+    parser.add_argument("--parallel", action="store_true", default=False,
+                        help="[experimental] Run jobs in parallel. "
+                             "Does not currently keep track of ResourceRequirements like the number of cores"
+                             "or memory and can overload this system")
+    envgroup = parser.add_mutually_exclusive_group()
+    envgroup.add_argument("--preserve-environment", type=Text, action="append",
+                        help="Preserve specific environment variable when "
+                        "running CommandLineTools.  May be provided multiple "
+                        "times.", metavar="ENVVAR", default=["PATH"],
                         dest="preserve_environment")
-
-    parser.add_argument("--preserve-entire-environment", action="store_true",
-                        help="Preserve entire parent environment when running CommandLineTools.",
-                        default=False,
+    envgroup.add_argument("--preserve-entire-environment", action="store_true",
+                        help="Preserve all environment variable when running "
+                        "CommandLineTools.", default=False,
                         dest="preserve_entire_environment")
 
     exgroup = parser.add_mutually_exclusive_group()
@@ -72,12 +72,12 @@ def arg_parser():  # type: () -> argparse.ArgumentParser
 
     parser.add_argument("--tmpdir-prefix", type=Text,
                         help="Path prefix for temporary directories",
-                        default="tmp")
+                        default=DEFAULT_TMP_PREFIX)
 
     exgroup = parser.add_mutually_exclusive_group()
     exgroup.add_argument("--tmp-outdir-prefix", type=Text,
                          help="Path prefix for intermediate output directories",
-                         default="tmp")
+                         default=DEFAULT_TMP_PREFIX)
 
     exgroup.add_argument("--cachedir", type=Text, default="",
                          help="Directory to cache intermediate workflow outputs to avoid recomputing steps.")
@@ -141,7 +141,7 @@ def arg_parser():  # type: () -> argparse.ArgumentParser
                          default=True, dest="strict")
 
     parser.add_argument("--skip-schemas", action="store_true",
-            help="Skip loading of schemas", default=True, dest="skip_schemas")
+            help="Skip loading of schemas", default=False, dest="skip_schemas")
 
     exgroup = parser.add_mutually_exclusive_group()
     exgroup.add_argument("--verbose", action="store_true", help="Default logging")
@@ -152,10 +152,22 @@ def arg_parser():  # type: () -> argparse.ArgumentParser
                         "timestamps to the errors, warnings, and "
                         "notifications.")
     parser.add_argument("--js-console", action="store_true", help="Enable javascript console output")
-    parser.add_argument("--user-space-docker-cmd",
+    dockergroup = parser.add_mutually_exclusive_group()
+    dockergroup.add_argument("--user-space-docker-cmd", metavar="CMD",
                         help="(Linux/OS X only) Specify a user space docker "
                         "command (like udocker or dx-docker) that will be "
                         "used to call 'pull' and 'run'")
+    dockergroup.add_argument("--singularity", action="store_true",
+                             default=False, help="[experimental] Use "
+                             "Singularity runtime for running containers. "
+                             "Requires Singularity v2.3.2+ and Linux with kernel "
+                             "version v3.18+ or with overlayfs support "
+                             "backported.")
+    dockergroup.add_argument("--no-container", action="store_false",
+                             default=True, help="Do not execute jobs in a "
+                             "Docker container, even when `DockerRequirement` "
+                             "is specified under `hints`.",
+                             dest="use_container")
 
     dependency_resolvers_configuration_help = argparse.SUPPRESS
     dependencies_directory_help = argparse.SUPPRESS
@@ -190,7 +202,7 @@ def arg_parser():  # type: () -> argparse.ArgumentParser
     parser.add_argument("--default-container",
                         help="Specify a default docker container that will be used if the workflow fails to specify one.")
     parser.add_argument("--no-match-user", action="store_true",
-                        help="Disable passing the current uid to 'docker run --user`")
+                        help="Disable passing the current uid to `docker run --user`")
     parser.add_argument("--disable-net", action="store_true",
                         help="Use docker's default networking for containers;"
                              " the default is to enable networking.")
@@ -235,8 +247,15 @@ def arg_parser():  # type: () -> argparse.ArgumentParser
     parser.add_argument("--overrides", type=str,
                         default=None, help="Read process requirement overrides from file.")
 
-    parser.add_argument("workflow", type=Text, nargs="?", default=None)
-    parser.add_argument("job_order", nargs=argparse.REMAINDER)
+    parser.add_argument("workflow", type=Text, nargs="?", default=None,
+            metavar='cwl_document', help="path or URL to a CWL Workflow, "
+            "CommandLineTool, or ExpressionTool. If the `inputs_object` has a "
+            "`cwl:tool` field indicating the path or URL to the cwl_document, "
+            " then the `workflow` argument is optional.")
+    parser.add_argument("job_order", nargs=argparse.REMAINDER,
+            metavar='inputs_object', help="path or URL to a YAML or JSON "
+            "formatted description of the required input values for the given "
+            "`cwl_document`.")
 
     return parser
 
