@@ -3,8 +3,9 @@ import os
 from typing import Callable as tCallable
 from typing import Any, Dict, Text, Tuple, Union
 
-from . import load_tool, main, workflow
+from . import load_tool, workflow
 from .argparser import get_default_args
+from .executors import SingleJobExecutor
 from .process import Process
 
 
@@ -36,11 +37,13 @@ class Factory(object):
     def __init__(self,
                  makeTool=workflow.defaultMakeTool,  # type: tCallable[[Any], Process]
                  # should be tCallable[[Dict[Text, Any], Any], Process] ?
-                 executor=main.single_job_executor,  # type: tCallable[...,Tuple[Dict[Text,Any], Text]]
+                 executor=None,  # type: tCallable[...,Tuple[Dict[Text,Any], Text]]
                  **execkwargs  # type: Any
                  ):
         # type: (...) -> None
         self.makeTool = makeTool
+        if executor is None:
+            executor = SingleJobExecutor()
         self.executor = executor
 
         kwargs = get_default_args()
