@@ -1,4 +1,5 @@
 from __future__ import absolute_import
+
 import codecs
 import functools
 import io
@@ -11,27 +12,28 @@ import stat
 import subprocess
 import sys
 import tempfile
-import datetime
+from abc import ABCMeta, abstractmethod
 from io import open
-from typing import (IO, Any, Callable, Dict, Iterable, List, MutableMapping, Text,
-                    Tuple, Union, cast)
+from threading import Lock
 
 import shellescape
+from typing import (IO, Any, Callable, Dict, Iterable, List, MutableMapping, Text,
+                    Union, cast)
 
-from .utils import copytree_with_merge, docker_windows_path_adjust, onWindows
-from . import docker
 from .builder import Builder
-from .docker_id import docker_vm_id
 from .errors import WorkflowException
-from .pathmapper import PathMapper, ensure_writable
-from .process import (UnsupportedRequirement, empty_subtree, get_feature,
+from .pathmapper import PathMapper
+from .process import (UnsupportedRequirement, get_feature,
                       stageFiles)
 from .utils import bytes2str_in_dicts
+from .utils import copytree_with_merge, onWindows
 
 _logger = logging.getLogger("cwltool")
 
 needs_shell_quoting_re = re.compile(r"""(^$|[\s|&;()<>\'"$@])""")
 
+job_output_lock = Lock()
+
 FORCE_SHELLED_POPEN = os.getenv("CWLTOOL_FORCE_SHELL_POPEN", "0") == "1"
 
 SHELL_COMMAND_TEMPLATE = """#!/bin/bash
@@ -267,7 +269,8 @@ class JobBase(object):
         if _logger.isEnabledFor(logging.DEBUG):
             _logger.debug(u"[job %s] %s", self.name, json.dumps(outputs, indent=4))
 
-        self.output_callback(outputs, processStatus)
+        with job_output_lock:
+            self.output_callback(outputs, processStatus)
 
         if self.stagedir and os.path.exists(self.stagedir):
             _logger.debug(u"[job %s] Removing input staging directory %s", self.name, self.stagedir)
@@ -312,56 +315,19 @@ class CommandLineJob(JobBase):
         self._execute([], env, rm_tmpdir=rm_tmpdir, move_outputs=move_outputs)
 
 
-class DockerCommandLineJob(JobBase):
+class ContainerCommandLineJob(JobBase):
+    __metaclass__ = ABCMeta
 
-    def add_volumes(self, pathmapper, runtime):
-        # type: (PathMapper, List[Text]) -> None
+    @abstractmethod
+    def get_from_requirements(self, r, req, pull_image, dry_run=False):
+        # type: (Dict[Text, Text], bool, bool, bool) -> Text
+        pass
 
-        host_outdir = self.outdir
-        container_outdir = self.builder.outdir
-        for src, vol in pathmapper.items():
-            if not vol.staged:
-                continue
-            if vol.target.startswith(container_outdir+"/"):
-                host_outdir_tgt = os.path.join(
-                    host_outdir, vol.target[len(container_outdir)+1:])
-            else:
-                host_outdir_tgt = None
-            if vol.type in ("File", "Directory"):
-                if not vol.resolved.startswith("_:"):
-                    runtime.append(u"--volume=%s:%s:ro" % (
-                        docker_windows_path_adjust(vol.resolved),
-                        docker_windows_path_adjust(vol.target)))
-            elif vol.type == "WritableFile":
-                if self.inplace_update:
-                    runtime.append(u"--volume=%s:%s:rw" % (
-                        docker_windows_path_adjust(vol.resolved),
-                        docker_windows_path_adjust(vol.target)))
-                else:
-                    shutil.copy(vol.resolved, host_outdir_tgt)
-                    ensure_writable(host_outdir_tgt)
-            elif vol.type == "WritableDirectory":
-                if vol.resolved.startswith("_:"):
-                    os.makedirs(host_outdir_tgt, 0o0755)
-                else:
-                    if self.inplace_update:
-                        runtime.append(u"--volume=%s:%s:rw" % (
-                            docker_windows_path_adjust(vol.resolved),
-                            docker_windows_path_adjust(vol.target)))
-                    else:
-                        shutil.copytree(vol.resolved, host_outdir_tgt)
-                        ensure_writable(host_outdir_tgt)
-            elif vol.type == "CreateFile":
-                if host_outdir_tgt:
-                    with open(host_outdir_tgt, "wb") as f:
-                        f.write(vol.resolved.encode("utf-8"))
-                else:
-                    fd, createtmp = tempfile.mkstemp(dir=self.tmpdir)
-                    with os.fdopen(fd, "wb") as f:
-                        f.write(vol.resolved.encode("utf-8"))
-                    runtime.append(u"--volume=%s:%s:rw" % (
-                        docker_windows_path_adjust(createtmp),
-                        docker_windows_path_adjust(vol.target)))
+    @abstractmethod
+    def create_runtime(self, env, rm_container, record_container_id, cidfile_dir,
+                       cidfile_prefix, **kwargs):
+        # type: (MutableMapping[Text, Text], bool, bool, Text, Text, **Any) -> List
+        pass
 
     def run(self, pull_image=True, rm_container=True,
             record_container_id=False, cidfile_dir="",
@@ -389,8 +355,7 @@ class DockerCommandLineJob(JobBase):
             try:
                 env = cast(MutableMapping[Text, Text], os.environ)
                 if docker_req and kwargs.get("use_container"):
-                    img_id = str(docker.get_from_requirements(
-                        docker_req, True, pull_image))
+                    img_id = str(self.get_from_requirements(docker_req, True, pull_image))
                 if img_id is None:
                     if self.builder.find_default_container:
                         default_container = self.builder.find_default_container()
@@ -401,99 +366,23 @@ class DockerCommandLineJob(JobBase):
                 if docker_req and img_id is None and kwargs.get("use_container"):
                     raise Exception("Docker image not available")
             except Exception as e:
-                _logger.debug("Docker error", exc_info=True)
+                container = "Singularity" if kwargs.get("singularity") else "Docker"
+                _logger.debug("%s error" % container, exc_info=True)
                 if docker_is_req:
                     raise UnsupportedRequirement(
-                        "Docker is required to run this tool: %s" % e)
+                        "%s is required to run this tool: %s" % (container, e))
                 else:
                     raise WorkflowException(
-                        "Docker is not available for this tool, try "
-                        "--no-container to disable Docker, or install "
+                        "{0} is not available for this tool, try "
+                        "--no-container to disable {0}, or install "
                         "a user space Docker replacement like uDocker with "
-                        "--user-space-docker-cmd.: %s" % e)
+                        "--user-space-docker-cmd.: {1}".format(container, e))
 
         self._setup(kwargs)
-
-        if user_space_docker_cmd:
-            runtime = [user_space_docker_cmd, u"run"]
-        else:
-            runtime = [u"docker", u"run", u"-i"]
-
-        runtime.append(u"--volume=%s:%s:rw" % (
-            docker_windows_path_adjust(os.path.realpath(self.outdir)),
-            self.builder.outdir))
-        runtime.append(u"--volume=%s:%s:rw" % (
-            docker_windows_path_adjust(os.path.realpath(self.tmpdir)), "/tmp"))
-
-        self.add_volumes(self.pathmapper, runtime)
-        if self.generatemapper:
-            self.add_volumes(self.generatemapper, runtime)
-
-        if user_space_docker_cmd:
-            runtime = [x.replace(":ro", "") for x in runtime]
-            runtime = [x.replace(":rw", "") for x in runtime]
-
-        runtime.append(u"--workdir=%s" % (
-            docker_windows_path_adjust(self.builder.outdir)))
-        if not user_space_docker_cmd:
-
-            if not kwargs.get("no_read_only"):
-                runtime.append(u"--read-only=true")
-
-            if kwargs.get("custom_net", None) is not None:
-                runtime.append(u"--net={0}".format(kwargs.get("custom_net")))
-            elif kwargs.get("disable_net", None):
-                runtime.append(u"--net=none")
-
-            if self.stdout:
-                runtime.append("--log-driver=none")
-
-            euid, egid = docker_vm_id()
-            if not onWindows():
-                # MS Windows does not have getuid() or geteuid() functions
-                euid, egid = euid or os.geteuid(), egid or os.getgid()
-
-            if kwargs.get("no_match_user", None) is False \
-                    and (euid, egid) != (None, None):
-                runtime.append(u"--user=%d:%d" % (euid, egid))
-
-        if rm_container:
-            runtime.append(u"--rm")
-
-        runtime.append(u"--env=TMPDIR=/tmp")
-
-        # spec currently says "HOME must be set to the designated output
-        # directory." but spec might change to designated temp directory.
-        # runtime.append("--env=HOME=/tmp")
-        runtime.append(u"--env=HOME=%s" % self.builder.outdir)
-        
-        # add parameters to docker to write a container ID file
-        if record_container_id:
-            if cidfile_dir != "":
-                if not os.path.isdir(cidfile_dir):
-                    _logger.error("--cidfile-dir %s error:\n%s", cidfile_dir,
-                                  cidfile_dir + " is not a directory or "
-                                  "directory doesn't exist, please check it first")
-                    exit(2)
-                if not os.path.exists(cidfile_dir):
-                    _logger.error("--cidfile-dir %s error:\n%s", cidfile_dir,
-                                  "directory doesn't exist, please create it first")
-                    exit(2)
-            else:
-                cidfile_dir = os.getcwd()
-            cidfile_name = datetime.datetime.now().strftime("%Y%m%d%H%M%S-%f")+".cid"
-            if cidfile_prefix != "":
-                cidfile_name = str(cidfile_prefix + "-" + cidfile_name)
-            cidfile_path = os.path.join(cidfile_dir, cidfile_name)
-            runtime.append(u"--cidfile=%s" % cidfile_path)
-
-        for t, v in self.environment.items():
-            runtime.append(u"--env=%s=%s" % (t, v))
-
+        runtime = self.create_runtime(env, rm_container, record_container_id, cidfile_dir, cidfile_prefix, **kwargs)
         runtime.append(img_id)
 
-        self._execute(
-            runtime, env, rm_tmpdir=rm_tmpdir, move_outputs=move_outputs)
+        self._execute(runtime, env, rm_tmpdir=rm_tmpdir, move_outputs=move_outputs)
 
 
 def _job_popen(
@@ -573,7 +462,7 @@ def _job_popen(
             stdin_path=stdin_path,
         )
         with open(os.path.join(job_dir, "job.json"), "wb") as f:
-            json.dump(job_description, codecs.getwriter('utf-8')(f), ensure_ascii=False) # type: ignore
+            json.dump(job_description, codecs.getwriter('utf-8')(f), ensure_ascii=False)  # type: ignore
         try:
             job_script = os.path.join(job_dir, "run_job.bash")
             with open(job_script, "wb") as f:
